//    Vodafone 190
//    Copyright (C) 2011 Paolo Iommarini
//    sakya_tg@yahoo.it
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#include "vodafonewidget.h"

VodafoneWidget::VodafoneWidget(QString username, QString password)
{    
    m_CurrentSimType = SimUnknown;
    m_Aborting = false;
    m_CurrentReply = NULL;
    DebugXml = false;
    m_Username = username;
    m_Password = password;

    m_CookieJar = new QNetworkCookieJar (this);
    m_Nam = new QNetworkAccessManager(this);
    m_Nam->setCookieJar(m_CookieJar);
} // VodafoneWidget

VodafoneWidget::~VodafoneWidget()
{
    delete m_CookieJar;
    m_CookieJar = NULL;
    delete m_Nam;
    m_Nam = NULL;
} // ~VodafoneWidget

void VodafoneWidget::SetCredentials(QString username, QString password)
{
    if ((!m_Username.isEmpty() && m_Username != username) || (!m_Password.isEmpty() && m_Password != password))
        Logout();
    m_Username = username;
    m_Password = password;
} // SetCredentials

bool VodafoneWidget::IsLoggedIn()
{
    bool result = false;
    QNetworkRequest request = PrepareRequest("https://widget.vodafone.it/190/trilogy/jsp/utility/checkUser.jsp");
    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "IsLoggedIn: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    QDomNode root = doc.firstChildElement("root");
    QDomNode logged = root.firstChildElement("logged-in");
    if (!logged.isNull() && logged.firstChild().nodeValue() == "true")
        result = true;

    QDomNode sessionID = root.firstChildElement("session-id");
    if (!sessionID.isNull())
        m_SessionID = sessionID.firstChild().nodeValue();

    //Read all the sims:
    if (result){
        m_Sims.clear();
        QDomNode user = root.firstChildElement("user");
        QDomNode sims = user.firstChildElement("all-sims");
        if (!sims.isNull()){
            QDomNodeList list = sims.childNodes();
            for (int i=0; i<list.count(); i++){
                QDomNode node = list.at(i);
                if (node.nodeName() == "msisdn")
                    m_Sims.append(node.firstChild().nodeValue());
            }
        }
        //m_Sims.append("123345");

        QDomNode sim = user.firstChildElement("current-msisdn");
        if (!sim.isNull())
            m_CurrentSim = sim.firstChild().nodeValue();

        QDomNode qualifiers = user.firstChildElement("all-user-qualifiers");
        if (!qualifiers.isNull()){
            QDomNodeList list = qualifiers.childNodes();
            for (int i=0; i<list.count(); i++){
                QDomNode node = list.at(i);
                QDomNode name = node.firstChildElement("qual-name");
                QDomNode value = node.firstChildElement("qual-value");
                if (!name.isNull() && !value.isNull() && name.firstChild().nodeValue() == "Tipo SIM"){
                    if (value.firstChild().nodeValue() == "ricaricabile")
                        m_CurrentSimType = SimRicaricabile;
                    else
                        m_CurrentSimType = SimContratto;
                    break;
                }
            }
        }
    }
    return result;
} // IsLoggedIn

bool VodafoneWidget::Login()
{
    QUrl url("https://widget.vodafone.it/190/trilogy/jsp/login.do");
    url.addQueryItem("username", m_Username);
    url.addQueryItem("password", m_Password);
    url.addQueryItem("cu_notrace", "true");
    QNetworkRequest request = PrepareRequest(url);

    QUrl newUrl;
    QString string = SynchRequest(&request, &newUrl);
    //The login process redirects through more than one page:
    while (newUrl.isValid()){        
        if (newUrl.path().endsWith("home.do")) // This redirect in not needed to login
            break;
        request = PrepareRequest(newUrl);        
        string = SynchRequest(&request, &newUrl);
    }

    return IsLoggedIn();
} // Login

void VodafoneWidget::Logout()
{
    QNetworkRequest request = PrepareRequest("https://widget.vodafone.it/190/trilogy/jsp/logout.do");
    SynchRequest(&request);
} // Logout

QNetworkRequest VodafoneWidget::PrepareRequest(QUrl url, bool vodafoneUserAgent)
{
    return PrepareRequest(url.toString(), vodafoneUserAgent);
} // PrepareRequest

QNetworkRequest VodafoneWidget::PrepareRequest(QString url, bool vodafoneUserAgent)
{
    QUrl tUrl(url);
    QNetworkRequest request(tUrl);
    if (vodafoneUserAgent)
        request.setRawHeader("User-Agent", "VODAFONE_DW");
    else
        request.setRawHeader("User-Agent", "Mozilla/5.0 (X11; U; Linux i686; pl-PL; rv:1.9.0.2) Gecko/20121223 Ubuntu/9.25 (jaunty) Firefox/3.8");
    return request;
} // PrepareRequest

QString VodafoneWidget::SynchRequest(QNetworkRequest* request, QUrl* redirectUrl)
{
    if (DebugXml)
        qDebug() << "SynchRequest: " << request->url().toString();

    QTime start;
    start.start();

    if (redirectUrl)
        redirectUrl->clear();
    QNetworkReply* reply = m_Nam->get(*request);
    m_CurrentReply = reply;
    reply->ignoreSslErrors();

    QEventLoop* loop = new QEventLoop();
    QObject::connect(m_Nam, SIGNAL(finished(QNetworkReply *)), loop, SLOT(quit()));
    loop->exec();
    QObject::disconnect(m_Nam, SIGNAL(finished(QNetworkReply *)), loop, SLOT(quit()));
    delete loop;
    if (m_Aborting){
        m_Aborting = false;
        m_CurrentReply = NULL;
        delete reply;
        return "";
    }

    if(reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() == 302 && redirectUrl)
        *redirectUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
    QByteArray bytes = reply->readAll();
    QString result(bytes);

    m_CurrentReply = NULL;
    delete reply;

    if (DebugXml){
        QTime elapsed(0, 0, 0, 0);
        elapsed = elapsed.addMSecs(start.elapsed());
        qDebug() << "SynchRequest: Time " << elapsed.toString("hh:mm:ss.z");
    }
    return result;
} // SynchRequest

float VodafoneWidget::GetCredito(QString* valuta)
{
    float result = 0;
    QNetworkRequest request = PrepareRequest("https://widget.vodafone.it/190/fast/mx/CreditoResiduoPush.do?hpfdtpri=y");
    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "GetCredito: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    int found = 0;
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "e"){
            QDomNode name = node.attributes().namedItem("n");
            if (name.nodeValue() == "CREDITO_RESIDUO"){
                QDomNode val = node.attributes().namedItem("v");
                QString credito = val.nodeValue();
                result = Decrypt(&credito, &m_SessionID).toFloat();
                found++;
            }else if (name.nodeValue() == "VALUTA"){
                QDomNode val = node.attributes().namedItem("v");
                *valuta = val.nodeValue();
                found++;
            }
        }
        if (found >= 2)
            break;
    }

    return result;
} // GetCredito

bool VodafoneWidget::GetInfoConto(InfoConto* info)
{
    QNetworkRequest request = PrepareRequest("https://widget.vodafone.it/190/ebwe/mx/PushInfoconto.do?hpfdtpri=y");
    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "GetInfoConto: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    int found = 0;
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "e"){
            QDomNode name = node.attributes().namedItem("n");
            if (name.nodeValue() == "TRAFFICO_NON_FATTURATO"){
                QDomNode val = node.attributes().namedItem("v");
                QString credito = val.nodeValue();
                info->TrafficoNonFatturato = credito.toFloat();
                found++;
            }else if (name.nodeValue() == "DATA_ULTIMA_FATTURAZIONE"){
                QDomNode val = node.attributes().namedItem("v");
                QString data = val.nodeValue();
                info->DataUltimaFatturazione = QDate::fromString(data, "yyyy-MM-dd hh:mm:ss");
                found++;
            }
        }
        if (found >= 2)
            break;
    }

    return true;
}

QList<Counter*>* VodafoneWidget::GetListaContatori(bool includeStandard)
{
    QList<Counter*>* result = new QList<Counter*>();

    QNetworkRequest request = PrepareRequest("https://widget.vodafone.it/190/ebwe/mx/Desktopwidgetconto.do", !includeStandard);
    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "GetListaContatori: " << string;

    QDomDocument doc("response");
    doc.setContent(string);
    if (GetReturnCode(doc) != "0")
        return result;

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "ProductItem"){
            Counter* c = new Counter();
            QDomNodeList clist = node.childNodes();
            int found = 0;
            for (int j=0; j<clist.count(); j++){
                QDomNode cnode = clist.at(j);
                if (cnode.nodeName() == "ProductType"){
                    c->Type = cnode.firstChild().nodeValue();
                    found++;
                }else if (cnode.nodeName() == "Classe"){
                    c->Class = cnode.firstChild().nodeValue();
                    found++;
                }else if (cnode.nodeName() == "ProductCode"){
                    c->Code = cnode.firstChild().nodeValue();
                    found++;
                }else if (cnode.nodeName() == "Descrizione"){
                    c->Description = cnode.firstChild().nodeValue();
                    found++;
                }
                if (found == 4)
                    break;
            }
            result->append(c);
        }
    }
    return result;
} // GetListaContatori

bool VodafoneWidget::GetDettaglioContatore(Counter* counter)
{
    QUrl url("https://widget.vodafone.it/190/ebwe/mx/DesktopwidgetContatoreDettagli.do");
    url.addQueryItem("ProductType", counter->Type);
    url.addQueryItem("Classe", counter->Class);
    url.addQueryItem("ProductCode", counter->Code);
    QNetworkRequest request = PrepareRequest(url, false);

    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "GetDettaglioContatore: " << string;

    QDomDocument doc("response");
    doc.setContent(string);
    if (GetReturnCode(doc) != "0")
        return false;

    counter->Soglie.clear();
    QDomNode root = doc.firstChildElement("root");

    QDomNode sogliaNode = root.firstChildElement("SogliaItem");
    while (!sogliaNode.isNull()){
        QDomNodeList list = sogliaNode.childNodes();
        int found = 0;
        SogliaItem soglia;
        for (int i=0; i<list.count(); i++){
            QDomNode node = list.at(i);
            if (node.nodeName() == "Soglia"){
                soglia.Soglia = GetCounterFloatValue(node.firstChild().nodeValue());
                found++;
            }else if (node.nodeName() == "DisplayType"){
                soglia.DisplayType = node.firstChild().nodeValue().toInt();
                found++;
            }else if (node.nodeName() == "TrafficoUtilizzato"){
                soglia.Utilizzato = GetCounterFloatValue(node.firstChild().nodeValue());
                found++;
            }else if (node.nodeName() == "TrafficoResiduo"){
                soglia.Residuo = GetCounterFloatValue(node.firstChild().nodeValue());
                found++;
            }else if (node.nodeName() == "DimensioneScatto"){
                soglia.DimensioneScatto = node.firstChild().nodeValue();
                found++;
            }else if (node.nodeName() == "UnitaContatore"){
                soglia.Unita = node.firstChild().nodeValue();
                found++;
            }else if (node.nodeName() == "UnitaScatto"){
                soglia.Scatto = GetCounterFloatValue(node.firstChild().nodeValue());
                found++;
            }else if (node.nodeName() == "Periodo"){
                soglia.Periodo = QDate::fromString(node.firstChild().nodeValue(), "dd/MM/yyyy");
                found++;
            }else if (node.nodeName() == "DataUtile"){
                soglia.DataUtile = QDate::fromString(node.firstChild().nodeValue(), "dd/MM/yyyy");
                found++;
            }

            if (found == 9)
                break;
        }
        counter->Soglie.append(soglia);

        sogliaNode = sogliaNode.nextSiblingElement("SogliaItem");
    }
    return true;
} // GetDettaglioContatore

QString VodafoneWidget::GetReturnCode(QDomDocument doc)
{
    QString res;

    QDomNode root = doc.firstChildElement("root");
    QDomNode returnCode = root.firstChildElement("ReturnCode");
    if (!returnCode.isNull())
        res = returnCode.firstChild().nodeValue();
    return res;
} // GetReturnCode

QString VodafoneWidget::GetStatus(QDomDocument doc, QString* errorMessage)
{
    QString res;

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    int found = 0;
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "e"){
            QDomNode name = node.attributes().namedItem("n");
            if (name.nodeValue() == "STATUS"){
                res = node.attributes().namedItem("v").nodeValue();
                found++;
            }else if (name.nodeValue() == "RETURNMSG"){
                if (errorMessage && node.firstChild().isCDATASection())
                    *errorMessage = node.firstChild().toCDATASection().data();
                found++;
            }
        }
        if (found == 2)
            break;
    }
    return res;
} // GetStatus

bool VodafoneWidget::PrepareMessage(QString receiver, QString message, QByteArray* captchaImage, QString* errorMessage)
{
    QUrl url("https://widget.vodafone.it/190/fsms/precheck.do?channel=VODAFONE_DW");
    QNetworkRequest request = PrepareRequest(url);
    SynchRequest(&request);

    url = QUrl("https://widget.vodafone.it/190/fsms/prepare.do?channel=VODAFONE_DW");
    url.addQueryItem("receiverNumber", receiver);
    url.addQueryItem("message", message);
    request = PrepareRequest(url);

    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "PrepareMessage: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    if (GetStatus(doc, errorMessage) != "1")
        return false;

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "e"){
            QDomNode name = node.attributes().namedItem("n");
            if (name.nodeValue() == "CODEIMG"){
                if (node.firstChild().isCDATASection()){
                    QString data = node.firstChild().toCDATASection().data();
                    *captchaImage = QByteArray::fromBase64(data.toLatin1());
                    break;
                }
            }
        }
    }
    return true;
} // PrepareMessage

bool VodafoneWidget::SendMessage(QString receiver, QString message, QString captcha, QString* errorMessage)
{
    QUrl url("https://widget.vodafone.it/190/fsms/send.do?channel=VODAFONE_DW");
    url.addQueryItem("verifyCode", captcha);
    url.addQueryItem("receiverNumber", receiver);
    url.addQueryItem("message", message);
    QNetworkRequest request = PrepareRequest(url);

    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "SendMessage: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    if (GetStatus(doc, errorMessage) != "1")
        return false;

    return true;
} // SendMessage

bool VodafoneWidget::GetSaldoPunti(SaldoPunti* saldo)
{
    QUrl url("https://widget.vodafone.it/190/jone/mx/SaldoPuntiPush.do?hpfdtpri=y");
    QNetworkRequest request = PrepareRequest(url);

    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "GetSaldoPunti: " << string;

    QDomDocument doc("response");
    doc.setContent(string);

    QDomNode root = doc.firstChildElement("root");
    QDomNodeList list = root.childNodes();
    int found = 0;
    for (int i=0; i<list.count(); i++){
        QDomNode node = list.at(i);
        if (node.nodeName() == "e"){
            QDomNode name = node.attributes().namedItem("n");
            if (name.nodeValue() == "SALDO_PUNTI"){
                QDomNode val = node.attributes().namedItem("v");
                QString punti = val.nodeValue();
                saldo->Punti = Decrypt(&punti, &m_SessionID).toFloat();
                found++;
            }else if (name.nodeValue() == "PUNTI_IN_SCADENZA"){
                QDomNode val = node.attributes().namedItem("v");
                QString punti = val.nodeValue();
                saldo->PuntiInScadenza = Decrypt(&punti, &m_SessionID).toFloat();
                found++;
            }else if (name.nodeValue() == "ANNO_ACCUMULO_PUNTI_IN_SCADENZA"){
                QDomNode val = node.attributes().namedItem("v");
                saldo->AnnoPuntiInScadenza = val.nodeValue().toInt();
                found++;
            }else if (name.nodeValue() == "DATA_AGGIORNAMENTO_SALDO"){
                QDomNode val = node.attributes().namedItem("v");
                QString data = val.nodeValue();
                data = data.left(data.indexOf("+"));
                saldo->Aggiornamento = QDate::fromString(data, "yyyy-MM-dd");
                found++;
            }else if (name.nodeValue() == "DATA_SCADENZA_PUNTI"){
                QDomNode val = node.attributes().namedItem("v");
                QString data = val.nodeValue();
                data = data.left(data.indexOf("+"));
                saldo->Scadenza = QDate::fromString(data, "yyyy-MM-dd");
                found++;
            }
        }
        if (found == 5)
            break;
    }

    return true;
} // GetSaldoPunti

QString VodafoneWidget::Decrypt(QString* crypted, QString* sessionID)
{
    QString cryptTable = " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\t!@#$%^&*()`\'-=[];,./?_+{}|:<>~";
    QString doubleQuote ="\"";
    QString lineFeed ="\n";
    int cryptLength = cryptTable.length() - 1;

    QString cryptedChar;
    int cryptedCharIndex = 0;
    QString decryptedChar;
    int tIndex = 0;
    bool check = false;
    QString result = "";
    QList<int> decryptTable;
    int sessionIdLength = sessionID->length();
    int cryptedLength = crypted->length();

    tIndex = 0;
    while (tIndex < sessionIdLength) {
        decryptTable.append(cryptTable.indexOf(sessionID->at(tIndex)));
        ++tIndex;
    }

    int pos = 0;
    tIndex = 0;
    while (pos < cryptedLength) {
        if (tIndex >= sessionIdLength)
            tIndex = 0;
        cryptedChar = crypted->at(pos);
        if ((cryptedCharIndex = cryptTable.indexOf(cryptedChar)) != -1) {
            if (check) {
                if (cryptedCharIndex != cryptLength) {
                    if (cryptedChar != "\'")
                        cryptedCharIndex = cryptedCharIndex + cryptLength;
                    else{
                        decryptedChar = doubleQuote;
                        cryptedCharIndex = -1;
                    }
                } else {
                    decryptedChar = lineFeed;
                    cryptedCharIndex = -1;
                }
                check = false;
            } else {
                if (cryptedCharIndex == cryptLength) {
                    check = true;
                    tIndex = (tIndex - 1);
                    decryptedChar = "";
                    cryptedCharIndex = -1;
                }
            }
        } else
            decryptedChar = cryptedChar;

        if (cryptedCharIndex != -1)
            decryptedChar = cryptTable.at(decryptTable[tIndex] ^ cryptedCharIndex);
        result.append(decryptedChar);
        ++pos;
        ++tIndex;
    }
    return result;
} // Decrypt

void VodafoneWidget::AbortRequest()
{
    if (m_CurrentReply){
        m_Aborting = true;
        m_CurrentReply->abort();
    }
} // AbortRequest

QStringList VodafoneWidget::GetSims()
{
    return m_Sims;
} // GetSims

QString VodafoneWidget::GetCurrentSim()
{
    return m_CurrentSim;
} // GetCurrentSim

VodafoneWidget::SimType VodafoneWidget::GetCurrentSimType()
{
    return m_CurrentSimType;
} // GetCurrentSimType

bool VodafoneWidget::SetCurrentSim(QString sim)
{
    QUrl url("https://widget.vodafone.it/190/trilogy/jsp/swapSim.do");
    url.addQueryItem("ty_sim", sim);
    QNetworkRequest request = PrepareRequest(url);

    QString string = SynchRequest(&request);
    if (DebugXml)
        qDebug() << "SetCurrentSim: " << string;

    if (!string.contains("parent.location.href=", Qt::CaseInsensitive))
        return false;

    m_CurrentSim = sim;
    return true;
}

float VodafoneWidget::GetCounterFloatValue(QString value)
{
    float res;

    QString tempValue = value.replace(",", ".");
    if (tempValue.contains(":")){
        QStringList ele = tempValue.split(":");
        res = ele.at(0).toInt() * 60;
        res += ele.at(1).toInt();
    }else{
        res = tempValue.toFloat();
    }
    return res;
}
